define("ember-engines/test-support/engine-resolver-for", ["exports", "ember-resolver"], function (_exports, _emberResolver) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = engineResolverFor;

  /* global require */

  /**
   * Gets the resolver class used by an Engine and creates an instance to be used
   * with test modules. Ex:
   *
   *   moduleForComponent('some-component', 'Integration Test', {
   *     resolver: engineResolverFor('ember-blog')
   *   });
   *
   * Uses the module found at `<engine-name>/resolver` as the class. If no module
   * exists at that path, then a default EmberResolver instance is created.
   *
   * You can optionally specify a modulePrefix in the event that the modulePrefix
   * differs from the engineName.
   *
   * @method engineResolverFor
   * @param {String} engineName
   * @param {String} [modulePrefix]
   * @return {Resolver}
   */
  function engineResolverFor(engineName, modulePrefix = engineName) {
    let Resolver;

    if (require.has("".concat(engineName, "/resolver"))) {
      Resolver = require("".concat(engineName, "/resolver")).default;
    } else {
      Resolver = _emberResolver.default;
    }

    return Resolver.create({
      namespace: {
        modulePrefix
      }
    });
  }
});