define("dummy/tests/integration/helpers/local-class-test", ["@ember/test-helpers", "ember-qunit", "qunit", "dummy/tests/helpers/register-module"], function (_testHelpers, _emberQunit, _qunit, _registerModule) {
  "use strict";

  (0, _qunit.module)('Integration | Helper | local-class', function (hooks) {
    (0, _emberQunit.setupRenderingTest)(hooks);
    (0, _qunit.test)('it passes through context-less local classes without processing', async function (assert) {
      var _temp;

      this.owner.register('service:makeup', (_temp = class MakeupServiceMock extends Ember.Service.extend(Ember.Evented) {
        constructor(...args) {
          super(...args);
          this.contextClassNamePrefix = 'some-class-name-prefix/';
        }

        resolveContext() {
          throw new Error('Should not be called');
        }

      }, _temp));
      this.moduleName = (0, _registerModule.default)({
        default: {
          foo: 'bar qux'
        }
      });
      await (0, _testHelpers.render)(Ember.HTMLBars.template(
      /*
        {{local-class 'foo' from=this.moduleName}}
      */
      {
        id: "CHLnBdmt",
        block: "{\"symbols\":[],\"statements\":[[1,[28,\"local-class\",[\"foo\"],[[\"from\"],[[23,0,[\"moduleName\"]]]]],false]],\"hasEval\":false}",
        meta: {}
      }));
      assert.equal(this.element.textContent.trim(), 'bar qux');
    });
    (0, _qunit.test)('it passes through context-less local classes and resolves context classes', async function (assert) {
      var _temp2;

      let flavor = 'light';
      this.owner.register('service:makeup', (_temp2 = class MakeupServiceMock extends Ember.Service.extend(Ember.Evented) {
        constructor(...args) {
          super(...args);
          this.contextClassNamePrefix = 'some-class-name-prefix/';
        }

        resolveContext(key) {
          return "context-".concat(flavor, "-").concat(key);
        }

      }, _temp2));
      this.moduleName = (0, _registerModule.default)({
        default: {
          foo: 'bar some-class-name-prefix/baz qux some-class-name-prefix/quax'
        }
      });
      await (0, _testHelpers.render)(Ember.HTMLBars.template(
      /*
        {{local-class 'foo' from=this.moduleName}}
      */
      {
        id: "CHLnBdmt",
        block: "{\"symbols\":[],\"statements\":[[1,[28,\"local-class\",[\"foo\"],[[\"from\"],[[23,0,[\"moduleName\"]]]]],false]],\"hasEval\":false}",
        meta: {}
      }));
      assert.equal(this.element.textContent.trim(), 'bar context-light-baz qux context-light-quax', 'resolves context classes');
      flavor = 'dark';
      this.owner.lookup('service:makeup').trigger('theme-change');
      Ember.run.begin();
      Ember.run.end();
      await (0, _testHelpers.settled)();
      assert.equal(this.element.textContent.trim(), 'bar context-dark-baz qux context-dark-quax', 'updates on `theme-change` event');
    });
  });
});