"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const broccoli_debug_1 = __importDefault(require("broccoli-debug"));
const broccoli_funnel_1 = __importDefault(require("broccoli-funnel"));
const broccoli_merge_trees_1 = __importDefault(require("broccoli-merge-trees"));
const broccoli_source_1 = require("broccoli-source");
const compare_versions_1 = require("compare-versions");
const addon_1 = __importDefault(require("ember-cli/lib/models/addon"));
const lodash_frompairs_1 = __importDefault(require("lodash.frompairs"));
const commands_1 = __importDefault(require("./commands"));
const options_1 = require("./lib/options");
const decorators_1 = require("./lib/utils/decorators");
const config_creator_1 = require("./plugins/broccoli/config-creator");
const extract_schema_1 = require("./plugins/broccoli/extract-schema");
const ember_css_modules_1 = __importDefault(require("./plugins/ember-css-modules"));
const preprocessor_registry_1 = require("./plugins/preprocessor-registry");
exports.addonName = require(`${__dirname}/../package`).name;
class EmberMakeupAddon extends addon_1.default {
    constructor() {
        super(...arguments);
        this.usages = {};
        this.debugTree = broccoli_debug_1.default.buildDebugCallback(this.name);
    }
    /**
     * Since `CoreObject` executes the `init` chain _during_ `super()`, the
     * `Addon` base class's `init` runs _before_ class instance property
     * assignments in the `EmberMakeupAddon` sub-class.
     *
     * Since `Addon#init` checks for the presence of `this.name`, we need to
     * assign the property here as opposed to just using a class instance property
     * assignment, like:
     *
     * ```ts
     * class EmberMakeupAddon extends Addon {
     *   name = 'ember-makeup';
     * }
     * ```
     *
     * @see https://github.com/ember-cli/ember-cli/blob/3af9f60cfe6e16caab0a972c7d25e8bb8017db26/lib/models/addon.js#L286-L288
     */
    init(...args) {
        this.name = exports.addonName;
        super.init(...args);
    }
    findThemePackages() {
        const keyword = 'ember-makeup-theme';
        const { dependencies = {}, devDependencies = {} } = this.project.pkg;
        // Would be using `.flatMap()`, but it's not supported in Node 8.
        // https://node.green/#ES2019-features-Array-prototype--flat--flatMap-
        // eslint-disable-next-line unicorn/prefer-flat-map
        const packages = []
            .concat(...[dependencies, devDependencies].map(Object.keys))
            .map(name => this.project.require(`${name}/package.json`))
            .filter(json => json.keywords && json.keywords.includes(keyword))
            .map(json => ({
            name: (json.makeup && json.makeup.name) || json.name,
            package: json
        }));
        if (packages.length === 0) {
            throw new Error(`Could not find a package with the '${keyword}' keyword.`);
        }
        return packages;
    }
    included(includer) {
        super.included(includer);
        this.findRootInstance();
        this.computeOptions(includer);
        this.findThemePackages();
    }
    includedCommands() {
        return commands_1.default;
    }
    computeOptions(includer) {
        if (this.makeupOptions)
            return;
        if (!includer.options && !this.app)
            throw new Error('Could not find parent options.');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const parentOptions = includer.options || this.app.options;
        this.makeupOptions = options_1.computeOptions(parentOptions.makeup);
    }
    findRootInstance() {
        if (this.parent.parent) {
            this.parentAddon = this.parent;
        }
        if (this.parentAddon) {
            const rootInstance = this.project.findAddonByName(this.name);
            if (!rootInstance) {
                throw new Error(`You need to install '${this.name}' in your project '${this.project.name()}', because '${this.parentAddon.name}' depends on it.`);
            }
            if ((!rootInstance.makeupOptions ||
                !rootInstance.makeupOptions.ignoreVersionCheck) &&
                (compare_versions_1.compare(rootInstance.pkg.version, this.pkg.version, '<') ||
                    compare_versions_1.compare(rootInstance.pkg.version, `${this.pkg.version.split('.')[0]}.*.*`, '>'))) {
                throw new Error(`The version of the root instance of ${this.name} (v${rootInstance.pkg.version}) is incompatible with the version used by '${this.parentAddon.name}' (v${this.pkg.version}).`);
            }
            this.rootInstance = rootInstance;
        }
        else {
            this.rootInstance = this;
        }
    }
    shouldIncludeChildAddon(childAddon) {
        const disabledAddons = ['ember-css-modules', 'ember-cli-sass'];
        if (disabledAddons.includes(childAddon.name)) {
            return false;
        }
        return super.shouldIncludeChildAddon(childAddon);
    }
    /**
     * Integrate with other CSS processors, like ember-cli-sass.
     *
     * @see https://github.com/ember-cli/ember-cli-preprocess-registry#addon-usage
     */
    setupPreprocessorRegistry(type, registry) {
        preprocessor_registry_1.register(this, type, registry);
    }
    /**
     * Integrate with ember-css-modules.
     *
     * @see https://github.com/salsify/ember-css-modules/blob/master/docs/PLUGINS.md#plugins
     */
    createCssModulesPlugin(parent) {
        this.computeOptions(parent);
        return new ember_css_modules_1.default(parent, this);
    }
    reportUsages(callsite, usages) {
        this.usages[callsite] = usages;
    }
    treeForAddon(tree) {
        const originalTree = this.debugTree(super.treeForAddon(tree), 'treeForAddon:input');
        // Only run for the root app.
        if (this.parentAddon)
            return originalTree;
        const configFile = config_creator_1.configCreatorJS(`${this.name}/config`, {
            options: this.makeupOptions,
            themePaths: lodash_frompairs_1.default(this.findThemePackages().map(theme => [
                theme.name,
                path_1.join('/', this.makeupOptions.pathPrefix, `${theme.name}.css`)
            ]))
        });
        const mergedTree = this.debugTree(new broccoli_merge_trees_1.default([originalTree, configFile], {
            annotation: 'ember-makeup:merge-config'
        }), 'treeForAddon:output');
        return mergedTree;
    }
    treeForPublic() {
        // Only run for the root app.
        if (this.parentAddon)
            return;
        return this.treeForConfig();
    }
    postprocessTree(type, tree) {
        // Only run for the root app and final processing with all files.
        // Only run, if compatibility layer is enabled.
        if (this.parentAddon ||
            type !== 'all' ||
            !this.makeupOptions.enableCompatibility)
            return tree;
        return new broccoli_merge_trees_1.default([tree, this.treeForCompatibility(tree)]);
    }
    treeForCompatibility(cssTree) {
        const configTree = this.treeForConfig();
        const schemaTree = new extract_schema_1.BroccoliExtractSchema(cssTree, {
            customPropertyPrefix: this.makeupOptions.customPropertyPrefix
        });
        const compatibilityTree = new config_creator_1.BroccoliConfigCreatorCSSCompatibility(configTree, schemaTree);
        return this.debugTree(compatibilityTree, 'treeForCompatibility');
    }
    treeForConfig() {
        // Only run for the root app.
        if (this.parentAddon)
            // This would happen, when this addon instance wrongfully call this method
            // on itself instead of the `rootInstance`.
            throw new Error('`treeForConfig` must not be called for addons.');
        const themePackages = this.findThemePackages();
        const themePackageSourceNodes = themePackages.map(themePackage => [
            themePackage.name,
            new broccoli_source_1.WatchedDir(path_1.dirname(this.project.resolveSync(`${themePackage.package.name}/package.json`)), { annotation: `ember-makeup:theme-source (${themePackage.name})` })
        ]);
        const tree = this.debugTree(new broccoli_funnel_1.default(new broccoli_merge_trees_1.default(themePackageSourceNodes.map(([themeName, source]) => config_creator_1.configCreatorCSS(source, {
            themeName,
            customPropertyPrefix: this.makeupOptions.customPropertyPrefix,
            contextClassNamePrefix: this.makeupOptions.contextClassNamePrefix
        }))), { destDir: this.makeupOptions.pathPrefix }), 'treeForConfig:output');
        return tree;
    }
}
__decorate([
    decorators_1.memoize
], EmberMakeupAddon.prototype, "findThemePackages", null);
__decorate([
    decorators_1.memoize
], EmberMakeupAddon.prototype, "treeForConfig", null);
exports.EmberMakeupAddon = EmberMakeupAddon;
exports.default = EmberMakeupAddon;
