"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const broccoli_persistent_filter_1 = __importDefault(require("broccoli-persistent-filter"));
const postcss_1 = require("postcss");
const util_1 = require("postcss-selector-parser/dist/util");
const postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
const config_key_1 = require("../../lib/config-key");
class BroccoliExtractSchema extends broccoli_persistent_filter_1.default {
    constructor(inputNode, { customPropertyPrefix }) {
        super(inputNode, {
            async: true,
            extensions: ['css'],
            targetExtension: 'makeup.json'
        });
        this.customPropertyPrefix = customPropertyPrefix;
        this.marker = `var(${config_key_1.serializeConfigKey(customPropertyPrefix)}`;
    }
    processString(contents, relativePath) {
        const root = postcss_1.parse(contents, { map: false, from: relativePath });
        const usages = [];
        root.walkDecls(decl => {
            if (!decl.value.includes(this.marker))
                return;
            if (decl.parent.type !== 'rule')
                throw decl.error('Declaration containing ember-makeup variable has to be nested in a rule.');
            usages.push({
                selectors: decl.parent.selectors,
                property: decl.prop,
                value: decl.value,
                tokens: this.extractTokens(decl.value)
            });
        });
        return JSON.stringify({ path: relativePath, usages });
    }
    extractTokens(value) {
        return postcss_value_parser_1.default(value).nodes.map(node => {
            if (node.type !== 'function' || node.value !== 'var')
                return node;
            const functionNode = node;
            if (functionNode.nodes.length !== 1 ||
                functionNode.nodes[0].type !== 'word')
                return node;
            const key = this.extractConfigKey(functionNode.nodes[0].value);
            if (!key)
                return node;
            functionNode.isVariableUsage = true;
            functionNode.key = key;
            return functionNode;
        });
    }
    extractConfigKey(value) {
        const key = util_1.unesc(value).slice(2); // trim off the leading `--`
        if (!key.startsWith(this.customPropertyPrefix))
            return undefined;
        return key.slice(this.customPropertyPrefix.length);
    }
}
exports.BroccoliExtractSchema = BroccoliExtractSchema;
