"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_1 = __importDefault(require("postcss"));
const postcss_value_parser_1 = require("postcss-value-parser");
const usage_1 = require("./usage");
const makeWordNode = (value) => ({
    type: 'word',
    value
});
const resolveVariableUsage = (theme, { key }) => {
    if (!(key in theme))
        throw new Error(`'${key}' is unknown`);
    return String(theme[key]);
};
const resolveTokens = (theme, tokens) => tokens.map(token => usage_1.isVariableUsage(token)
    ? makeWordNode(resolveVariableUsage(theme, token))
    : token);
const isUsageContexless = (theme, { tokens }) => tokens.some(usage => usage_1.isVariableUsage(usage) && usage.key in theme.contextless);
const batchUsages = (theme, usages) => usages.reduce(({ contextless, contextual }, usage) => {
    const batch = isUsageContexless(theme, usage) ? contextless : contextual;
    for (const selector of usage.selectors) {
        if (!batch[selector])
            batch[selector] = {};
        batch[selector][usage.property] = usage.tokens;
    }
    return { contextless, contextual };
}, {
    contextless: {},
    contextual: {}
});
function generateCompatibilityCSS(theme, usages) {
    const root = postcss_1.default.root();
    const { contextless, contextual } = batchUsages(theme, usages);
    for (const [selector, decls] of Object.entries(contextless)) {
        root.append(postcss_1.default.rule({
            selector,
            nodes: Object.entries(decls).map(([prop, tokens]) => postcss_1.default.decl({
                prop,
                value: postcss_value_parser_1.stringify(resolveTokens(theme.contextless, tokens))
            }))
        }));
    }
    for (const [selector, decls] of Object.entries(contextual)) {
        for (const [contextName, configValues] of Object.entries(theme.contextual)) {
            try {
                root.append(postcss_1.default.rule({
                    selector: `:context(${contextName}) ${selector}`,
                    nodes: Object.entries(decls).map(([prop, tokens]) => postcss_1.default.decl({
                        prop,
                        value: postcss_value_parser_1.stringify(resolveTokens(configValues, tokens))
                    }))
                }));
            }
            catch (error) {
                error.message = `'${selector}': ${error.message}`;
                throw error;
            }
        }
    }
    return root;
}
exports.generateCompatibilityCSS = generateCompatibilityCSS;
